// Graph.cpp : implementation file
//

#include "stdafx.h"
//#include "ECG.h"
#include "Graph.h"

#define CLASSNAME    _T("GraphCtrl")  // Window class name

// CGraph

IMPLEMENT_DYNAMIC(CGraph, CWnd)

CGraph::CGraph()
{
	RegisterWindowClass();
	m_bSetup = FALSE;
}

CGraph::~CGraph()
{
	for ( int j=0; j<m_arrSeries.GetSize(); j++)
		delete m_arrSeries[j];
	m_arrSeries.RemoveAll();
}

BOOL CGraph::RegisterWindowClass()
{
    WNDCLASS wndcls;
    HINSTANCE hInst = AfxGetInstanceHandle();

    if (!(::GetClassInfo(hInst, CLASSNAME, &wndcls)))
    {
        // otherwise we need to register a new class

        wndcls.style            = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
        wndcls.lpfnWndProc      = ::DefWindowProc;
        wndcls.cbClsExtra       = wndcls.cbWndExtra = 0;
        wndcls.hInstance        = hInst;
        wndcls.hIcon            = NULL;
        wndcls.hCursor          = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
        wndcls.hbrBackground    = (HBRUSH) (COLOR_3DFACE + 1);
        wndcls.lpszMenuName     = NULL;
        wndcls.lpszClassName    = CLASSNAME;

        if (!AfxRegisterClass(&wndcls))
        {
            AfxThrowResourceException();
            return FALSE;
        }
    }

    return TRUE;
}


BEGIN_MESSAGE_MAP(CGraph, CWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()



// CGraph message handlers



void CGraph::OnPaint()
{
	if (!m_bSetup)
		Setup( GetSysColor(15), 0x00ff00);

	CPaintDC dc(this); // device context for painting
	CDC *lpDC = &m_dcBuffer;

	CRect rcRect, rcWndRect;
	GetClientRect( &rcWndRect );
	rcRect = rcWndRect;
	rcRect.right += rcRect.Width();
	rcRect.bottom += rcRect.Height();

	int i;
	CRect rcSub = rcRect;

	COLORREF clrDelta = ColorDelta( m_clrBack, -20, -20, 0 );
	for ( i = rcRect.top; i < rcRect.bottom; i+=4 )
	{
		CBrush myBrush( ColorMid( m_clrBack, clrDelta, i*255/rcRect.bottom ) );
		rcSub.top = i;
		rcSub.bottom = i+20;
		lpDC->FillRect( rcSub, &myBrush );
	}

	m_nTotalWidth = 0;
	for ( int j=0; j<m_arrSeries.GetSize(); j++)
	{
		m_nTotalWidth = max( m_nTotalWidth, (int)m_arrSeries[j]->m_arrData.GetSize() ); 
	}

	for ( int j=0; j<(int)m_arrSeries.GetSize(); j++)
	{
		CGraphSeries* pSeries = m_arrSeries[j];
		CPen penLine(PS_SOLID, pSeries->m_nWidth, pSeries->m_clrColor);

		HPEN hOldPen = (HPEN)lpDC->SelectObject( penLine );

		if ( m_arrSeries[j]->m_nStyle == 0 )
		{
			for ( i = pSeries->Min(); i < pSeries->Max(); i++ )
			{
				( i == pSeries->Min() ) ?
					lpDC->MoveTo( GetPoint( j, i ) ) :
					lpDC->LineTo( GetPoint( j, i ) );
			}
		} else
		if ( m_arrSeries[j]->m_nStyle == 1 )
		{
			for ( i = pSeries->Min(); i < pSeries->Max(); i++ )
			{
				CPoint cp( GetPoint( j, i ) );
				lpDC->MoveTo( cp );
				cp.y = rcRect.bottom;
				lpDC->LineTo( cp );
			}
		} else {
			CBrush myBrush( pSeries->m_clrColor );
			for ( i = pSeries->Min(); i < pSeries->Max(); i++ )
			{
				CPoint cp( GetPoint( j, i ) );

				CRect rc(cp.x, cp.y, cp.x+2, cp.y+2);
				lpDC->FillRect(&rc, &myBrush);
			}
		}

		lpDC->SelectObject( hOldPen );
	}

	if (lpDC != &dc)
	{
		dc.SetStretchBltMode(HALFTONE);
		dc.SetBrushOrg(0, 0);
		dc.StretchBlt(0, 0, rcWndRect.Width(), rcWndRect.Height(), lpDC, 0, 0, rcRect.Width(), rcRect.Height(), SRCCOPY );
	}
	//dc.BitBlt( 0, 0, rcWndRect.Width(), rcWndRect.Height(), lpDC, 0, 0, SRCCOPY);
	//BitBlt( 0, 0, rcRect.Width(), rcRect.Height(), lpDC, 0, 0, SRCCOPY);
	//GdiFlush();
}

void CGraph::Setup(COLORREF clrBack, COLORREF clrLine)
{
	m_bSetup = TRUE;
	m_clrBack = clrBack;
	CRect rcRect;
	GetClientRect( &rcRect );

	m_nDCWidth = rcRect.Width()*2;
	m_nDCHeight = rcRect.Height()*2;

	CDC *lpDC = GetDC();
	m_dcBuffer.CreateCompatibleDC( lpDC );
	m_bmpBuffer.CreateBitmap ( m_nDCWidth, m_nDCHeight, 1, 32, NULL);
	m_dcBuffer.SelectObject( m_bmpBuffer );
}

CPoint CGraph::GetPoint( int series, int n )
{
	CGraphSeries* pSeries = m_arrSeries[series];
	float fVal = pSeries->Get(n);

	//int nCount = (int)pSeries->m_arrData.GetSize();
	int nX = n * m_nDCWidth / m_nTotalWidth;

	fVal = max(pSeries->m_fMin, min(fVal, pSeries->m_fMax));

	fVal = ( fVal - pSeries->m_fMin )/( pSeries->m_fMax - pSeries->m_fMin );
	fVal = 1.0f - fVal;

	return CPoint( nX, (int)(fVal*m_nDCHeight) );
}

CGraphSeries* CGraph::GetSeries( int n )
{
	if ( n < m_arrSeries.GetSize() )
		return m_arrSeries[n];
	
	CGraphSeries* pAux = NULL;
	int i = (int)m_arrSeries.GetSize();
	m_arrSeries.SetSize(n+1);
	for (; i<=n; i++)
	{
		pAux = new CGraphSeries();
		m_arrSeries.SetAt( i, pAux );
	}
	return pAux;
}

COLORREF CGraph::ColorMid( COLORREF a, COLORREF b, int l )
{
	int ra = a >> 16;				int rb = b >> 16;
	int ga = (a >> 8) & 0xff;		int gb = (b >> 8) & 0xff;
	int ba = a & 0xff;				int bb = b & 0xff;

	ra += ((rb-ra)*l)>>8;
	ga += ((gb-ga)*l)>>8;
	ba += ((bb-ba)*l)>>8;
	return (ra << 16) | (ga << 8) | ba;
}

COLORREF CGraph::ColorDelta( COLORREF clr, int r, int g, int b )
{
	int ba = clr >> 16;
	int ga = (clr >> 8) & 0xff;
	int ra = clr & 0xff;		
	ra += r;
	ga += g;
	ba += b;
	if ( ra > 255 ) ra = 255;
	if ( ra < 0 ) ra = 0;
	if ( ga > 255 ) ga = 255;
	if ( ga < 0 ) ga = 0;
	if ( ba > 255 ) ba = 255;
	if ( ba < 0 ) ba = 0;
	return (ba << 16) | (ga << 8) | ra;
}